import * as interfaces from '../interfaces/index.js';
import * as plugins from '../plugins.js';
import { WorkerEvent } from '../classes.workerevent.js';

declare const fetch: plugins.cloudflareTypes.Fetcher['fetch'];

export const originResponder: interfaces.TRequestResponser = async (eventArg: WorkerEvent) => {
  const originResponse: any = await fetch(eventArg.request);
  // lets capture status
  if (originResponse.status > 399) {
    return;
  }

  const headers = new Headers();
  for (const kv of originResponse.headers.entries()) {
    headers.append(kv[0], kv[1]);
  }
  headers.append('SERVEZONE_ROUTE', 'LOSSLESS_EDGE_FASTORIGIN');

  const responsePassThroughStream = new TransformStream();
  originResponse.body.pipeTo(responsePassThroughStream.writable);

  // response

  const responseForClient = new Response(responsePassThroughStream.readable, {
    ...originResponse,
    headers,
  });

  eventArg.setResponse(responseForClient);
};
